﻿namespace Operátory
{

    public struct Komplex
    {
        public double Re { get; set; }
        public double Im { get; set; }
        public Komplex(double re, double im = 0) => (Re, Im) = (re, im);
        public override string ToString() => $"{Re} + {Im}.i";
        public static readonly Komplex i = new Komplex(0, 1);
        public readonly void Deconstruct(out double re, out double im) => (re, im) = (Re, Im);
        public static Komplex operator +(Komplex z) => z;
        public static Komplex operator -(Komplex z) => new Komplex(-z.Re, -z.Im);
        public static Komplex operator +(Komplex z, Komplex w) => new Komplex(z.Re + w.Re, z.Im + w.Im);
        public static Komplex operator *(Komplex z, Komplex w)=> new Komplex(z.Re*w.Re - z.Im*w.Im, z.Re*w.Im + z.Im*w.Re);
        public static Komplex conj(Komplex z) => new Komplex(z.Re, -z.Im);
        public static Komplex operator/(Komplex z, double x)=>new Komplex(z.Re/x, z.Im/x);
        public static Komplex operator /(Komplex z, Komplex w) => (z * conj(w)) / (w * conj(w)).Re;
        public static bool operator ==(Komplex z, Komplex w)=> (z.Re == w.Re) && (z.Im == w.Im);
        public static bool operator !=(Komplex z, Komplex w) => !(z == w);
        public static implicit operator Komplex(double x) => new Komplex(x);
        public static explicit operator double(Komplex z)
        {
            return z.Im != 0.0 ? throw new ArgumentException("Přetypování komplexního čísla na reálné vyžaduje nulovou imaginární část") 
                               : z.Re;
        }

        public double this[int i]
        {
            get
            {
                return i switch
                {
                    1 => Re,
                    2 => Im,
                    _ => throw new IndexOutOfRangeException("Index mimo meze"),
                };
            }
            set
            {
                switch(i)
                {
                    case 1:
                        Re = value;
                        break;
                    case 2:
                        Im = value;
                        break;
                    default: throw new IndexOutOfRangeException("Index mimo meze");
                }
            }
        }
    }
    internal class Program
    {
        private static void Main(string[] args)
        {
            Console.WriteLine(Komplex.i);
            Komplex z = new Komplex(0.1, 5.21);
            Console.WriteLine(z);
            double x, y;
            (x, y) = z;
            z = (Komplex)11;
            z[1] = z[2];
        }
    }
}